/*
 * Copyright 2012 Thomas Bocek
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package net.tomp2p.simgrid.example;

import java.io.IOException;

import net.tomp2p.futures.BaseFutureAdapter;
import net.tomp2p.futures.FutureBootstrap;
import net.tomp2p.futures.FutureDHT;
import net.tomp2p.p2p.Peer;
import net.tomp2p.peers.Number160;
import net.tomp2p.peers.PeerAddress;
import net.tomp2p.simgrid.SimGridTomP2P;
import net.tomp2p.simgrid.Simulation;
import net.tomp2p.storage.Data;

import org.simgrid.msg.Process;

public class BoostrapPutGet implements Simulation
{
	public static void main(String[] args) throws IOException
	{
		//set the callback
		SimGridTomP2P.setSimulation(new BoostrapPutGet());
		//start SimGrid
		SimGridTomP2P.main(args);
	}
	@Override
	public void simulate(Peer peer, Process process, String[] args) throws Exception
	{
		Number160 bootstrapTo = Number160.createHash(args[0]);
		PeerAddress bootstrapPeer = new PeerAddress(bootstrapTo);
		bootstrap(peer.getPeerID(), bootstrapTo, peer, bootstrapPeer);
	}
	
	private void doDHT(Peer peer) throws IOException
	{
		if(peer.getPeerID().hashCode() % 5 == 0)
		{
			FutureDHT futureDHT = peer.put(Number160.createHash("storeme"), new Data(new String("I stored my host name "+peer.getPeerID())));
			futureDHT.addListener(new BaseFutureAdapter<FutureDHT>()
			{
				@Override
				public void operationComplete(FutureDHT future) throws Exception
				{
					System.out.println("put "+future.isSuccess());
				}
			});
		}
		else
		{
			FutureDHT futureDHT = peer.get(Number160.createHash("storeme"));
			futureDHT.addListener(new BaseFutureAdapter<FutureDHT>()
			{
				@Override
				public void operationComplete(FutureDHT future) throws Exception
				{
					System.out.println("get "+future.isSuccess());
				}
			});
		}
	}
	
	private void bootstrap(Number160 peerID,Number160 bootstrapTo, final Peer peer, PeerAddress bootstrapPeer)
	{
		if(!peerID.equals(bootstrapTo))
		{
			System.out.println("bootstrap from " + peerID + " to " + bootstrapTo);
			peer.bootstrap(bootstrapPeer).addListener(new BaseFutureAdapter<FutureBootstrap>()
			{
				@Override
				public void operationComplete(FutureBootstrap future) throws Exception
				{
					System.out.println("bootstrap success " + future.isSuccess());
					doDHT(peer);
				}
			});
		}
	}
}